@@ -151,7 +151,8 @@ class TenjinEntry(Entry):
     def _execute_create(self, context, ntimes):
         filename = self.template_filename
         for i in xrange(ntimes):
-            engine = tenjin.Engine(cache=True)
+            cache = tenjin.MarshalCacheStorage()
+            engine = tenjin.Engine(cache=cache)
             output = engine.render(filename, context)
         return output
 
@@ -962,9 +963,10 @@ def load_context_data(datafile):
     if encoding: s = s.decode(encoding)
     context = {}
     exec(s, globals(), context)
-    if 'items' in context and 'items2' in context:
-        k = mode == 'dict' and 'items' or 'items2'
-        context['stocks'] = context[k]
+    #if 'items' in context and 'items2' in context:
+    #    k = mode == 'dict' and 'items' or 'items2'
+    #    context['stocks'] = context[k]
+    context['stocks'] = context['items']
     return context
 
 
