@@ -189,7 +189,7 @@ def file_test_users_guide_py(c):
 
 
 @recipe
-@product(testdir+'/test_users_guide.py')
+@product(testdir+'/test_users-guide.py')
 @ingreds('users-guide.txt')
 def file_test_users_guide_py(c):
     ##
@@ -233,6 +233,8 @@ from glob import glob
 from oktest import ok, run
 
 python3 = sys.version_info[0] == 3
+PYPY    = hasattr(sys, 'pypy_version_info')
+JYTHON  = hasattr(sys, 'JYTHON_JAR')
 
 try:    # Python 2.6 or later
     from subprocess import Popen, PIPE
@@ -252,7 +254,8 @@ class UsersGuideTest(object):
     CWD = os.getcwd()
 
     def before(self):
-        sys.stdout.write('\n** test_%s: (' % self.__name__)
+        #sys.stdout.write('\n** test_%s: (' % self.__name__)
+        sys.stdout.write(' (')
         os.chdir(self.DIR + '/test_' + self.__name__)
         for x in glob('*.cache') + glob('views/*.cache'):
             os.unlink(x)
@@ -263,12 +266,23 @@ class UsersGuideTest(object):
                     import shutil
                     shutil.copy(fname, 'my_template.py')
                     break
+        if PYPY:
+            if self.__name__ == 'syntaxerr':
+                s = (
+                    "$ pytenjin -z syntaxerr.pyhtml\n"
+                    "syntaxerr.pyhtml:5:8: invalid syntax\n"
+                    "  5:         else\n"
+                    "            ^\n"
+                    )
+                f = open('result.output', 'w'); f.write(s); f.close()
+                del s
 
     def after(self):
         os.chdir(self.CWD)
 
     def _test(self):
-        for fname in glob('result*.output'):
+        result_files = glob('result*.output')
+        for fname in result_files:
             sys.stdout.write(' %s' % fname)
             result = open(fname).read()
             command, expected = re.split(r'\n', result, 1)
@@ -284,6 +298,18 @@ class UsersGuideTest(object):
                 actual = re.sub(r'file=.*?/test_logging/', "file='/home/user/", actual)
             else:
                 actual = os.popen(command).read()
+                if self.__name__ == 'm17n':
+                    expected = re.sub(r'timestamp: \d+(\.\d+)?', 'timestamp: 0.0', expected)
+                    actual   = re.sub(r'timestamp: \d+(\.\d+)?', 'timestamp: 0.0', actual)
+            if self._testMethodName == 'test_nested':
+                expected = re.sub(r'[ \t]*\#.*', '', expected)
+            ok (actual) == expected
+        if not result_files:
+            fname = glob('*main*.py')[0]
+            command = sys.executable + " " + fname
+            actual = os.popen(command).read()
+            fname = glob('*.expected')[0]
+            f = open(fname); expected = f.read(); f.close()
             ok (actual) == expected
         sys.stdout.write(' )')
 """)
@@ -338,7 +364,8 @@ class ExamplesTest(object):
     CWD = os.getcwd()
 
     def before(self):
-        sys.stdout.write('\n** test_%s: (' % self.__name__)
+        #sys.stdout.write('\n** test_%s: (' % self.__name__)
+        sys.stdout.write(' (')
         os.chdir(self.DIR + '/' + self.__name__)
         for x in glob('views/*.cache'):
             os.unlink(x)
