@@ -1,6 +1,6 @@
 ###
-### $Release: 1.0.2 $
-### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
+### $Release: 1.1.0 $
+### $Copyright: copyright(c) 2007-2012 kuwata-lab.com all rights reserved. $
 ###
 
 
@@ -14,7 +14,7 @@ def main(verbose):
     basenames = [
         "test_template",
         "test_engine",
-        "test_preprocess",
+        "test_preprocessor",
         "test_safe",
         "test_htmlhelper",
         "test_main",
@@ -32,6 +32,8 @@ def main(verbose):
     if python3:
         basenames.remove("test_encoding")
         basenames.remove("test_gae")
+    if python2 and sys.version_info[1] <= 4:
+        basenames.remove("test_gae")
 
     if verbose:
 
@@ -60,7 +62,8 @@ def main(verbose):
                     if type(klass) == type:
                         test_classes.append(klass)
         import oktest
-        oktest.run(*test_classes)
+        kwd = {'style': 'simple'}
+        oktest.run(*test_classes, **kwd)
 
 
 
