@@ -1,9 +1,9 @@
 ###
-### $Release: 1.0.2 $
-### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
+### $Release: 1.1.0 $
+### $Copyright: copyright(c) 2007-2012 kuwata-lab.com all rights reserved. $
 ###
 
-from oktest import ok, not_ok, run, spec
+from oktest import ok, not_ok, run, spec, test
 from oktest.helper import dummy_file
 import sys, os, re, time, marshal, shutil
 from glob import glob
@@ -17,6 +17,8 @@ from tenjin.helpers import *
 
 lvars = "_extend=_buf.extend;_to_str=to_str;_escape=escape; "
 
+JYTHON = hasattr(sys, 'JYTHON_JAR')
+
 
 def _convert_data(data, lang='python'):
     if isinstance(data, dict):
@@ -93,9 +95,9 @@ class EngineTest(object):
     #code = TestCaseHelper.generate_testcode(__file__)
     #exec(code)
     datalist = TestCaseHelper.load_testdata(__file__)
-    testdata = dict([ (d['name'], d) for d in datalist ])
-    _convert_data(testdata, lang='python')
-    data = testdata['basic']
+    _testdata = dict([ (d['name'], d) for d in datalist ])
+    _convert_data(_testdata, lang='python')
+    data = _testdata['basic']
     for d in data['templates']:
         d['filename'] = d['filename'].replace('.xxhtml', '.pyhtml')
     templates = dict([(hash['filename'], hash['content']) for hash in data['templates']])
@@ -105,13 +107,13 @@ class EngineTest(object):
 
     def before(self):
         tenjin.Engine.cache.clear()
-    #    testdata = EngineTest.testdata['basic']
+    #    testdata = EngineTest._testdata['basic']
     #    for hash in testdata['templates']:
     #        write_file(hash['filename'], hash['content'])
 
 
     #def after(self):
-    #    for hash in EngineTest.testdata['basic']['templates']:
+    #    for hash in EngineTest._testdata['basic']['templates']:
     #        filename = hash['filename']
     #        for fname in [filename, filename+'.cache', filename+'.marshal']:
     #            if os.path.exists(fname):
@@ -126,7 +128,7 @@ class EngineTest(object):
 
     def _test_basic(self):
         try:
-            testdata = EngineTest.testdata['basic']
+            testdata = EngineTest._testdata['basic']
             for hash in testdata['templates']:
                 write_file(hash['filename'], hash['content'])
             #
@@ -148,9 +150,9 @@ class EngineTest(object):
                 output = engine.render(tplname, context, layout=False)
             ok (output) == expected
         finally:
-            filenames = [ hash['filename'] for hash in EngineTest.testdata['basic']['templates'] ]
+            filenames = [ hash['filename'] for hash in EngineTest._testdata['basic']['templates'] ]
             _remove_files(filenames)
-            #for hash in EngineTest.testdata['basic']['templates']:
+            #for hash in EngineTest._testdata['basic']['templates']:
             #    filename = hash['filename']
             #    for fname in [filename, filename+'.cache', filename+'.marshal']:
             #        if os.path.exists(fname):
@@ -221,7 +223,7 @@ class EngineTest(object):
 
 
     def test_capture_and_echo(self):
-        hash = EngineTest.testdata['test_capture_and_echo']
+        hash = EngineTest._testdata['test_capture_and_echo']
         layout = hash['layout']
         content = hash['content']
         expected = hash['expected']
@@ -281,7 +283,7 @@ class EngineTest(object):
                 d.__exit__(*sys.exc_info())
 
     def test_captured_as(self):
-        hash = EngineTest.testdata['test_captured_as']
+        hash = EngineTest._testdata['test_captured_as']
         files = ( ('content.pyhtml',      hash['content']),
                   ('customlayout.pyhtml', hash['customlayout']),
                   ('baselayout.pyhtml',   hash['baselayout']),
@@ -299,7 +301,7 @@ class EngineTest(object):
 
 
     def test_local_layout(self):
-        hash = EngineTest.testdata['test_local_layout']
+        hash = EngineTest._testdata['test_local_layout']
         context = hash['context']
         names = ['layout_html', 'layout_xhtml', 'content_html']
         def fname(base):
@@ -334,7 +336,7 @@ class EngineTest(object):
 
 
     def test_cachefile(self):
-        data = EngineTest.testdata['test_cachefile']
+        data = EngineTest._testdata['test_cachefile']
         filenames = { 'layout': 'layout.pyhtml',
                       'page': 'account_create.pyhtml',
                       'form': 'account_form.pyhtml',
@@ -353,21 +355,26 @@ class EngineTest(object):
             ok (output) == expected
             for fname in cache_filenames: not_ok (fname).exists()
             ## marshal caching
-            props['cache'] = True
-            engine = tenjin.Engine(**props)
-            output = engine.render(':create', context)
-            ok (output) == expected
-            if   python2:  nullbyte = '\0'
-            elif python3:  nullbyte = '\0'.encode('ascii')
-            for fname in cache_filenames:
-                ok (fname).exists()                         # file created?
-                s = read_file(fname, 'rb')                       # read binary file
-                ok (s.find(nullbyte)) >= 0           # binary file?
-                f = lambda: marshal.load(open(fname, 'rb'))
-                ok (f).not_raise()                           # marshal?
-            engine = tenjin.Engine(**props)
-            output = engine.render(':create', context)
-            ok (output) == expected               # reloadable?
+            if not JYTHON:
+                props['cache'] = tenjin.MarshalCacheStorage()
+                engine = tenjin.Engine(**props)
+                output = engine.render(':create', context)
+                ok (output) == expected
+                if   python2:  nullbyte = '\0'
+                elif python3:  nullbyte = '\0'.encode('ascii')
+                for fname in cache_filenames:
+                    ok (fname).exists()               # file created?
+                    s = read_file(fname, 'rb')        # read binary file
+                    ok (s.find(nullbyte)) >= 0        # binary file?
+                    f = open(fname, 'rb')
+                    fn = lambda: marshal.load(f)
+                    try:
+                        ok (fn).not_raise()           # marshal?
+                    finally:
+                        f.close()
+                engine = tenjin.Engine(**props)
+                output = engine.render(':create', context)
+                ok (output) == expected               # reloadable?
             #
             for fname in glob('*.pyhtml.cache'): os.unlink(fname)
             for fname in cache_filenames:
@@ -386,9 +393,15 @@ class EngineTest(object):
                     ok (s.find(nullbyte)) < 0        # text file? (pickle protocol ver 2)
                 elif python3:
                     ok (s.find(nullbyte)) >= 0       # binary file? (pickle protocol ver 3)
-                f = lambda: Pickle.load(open(fname, 'rb'))
-                ok (f).not_raise(ValueError)
-                pickle.load(open(fname, 'rb'))
+                f = open(fname, 'rb')
+                fn = lambda: Pickle.load(f)
+                try:
+                    ok (fn).not_raise(ValueError)
+                finally:
+                    f.close()
+                f = open(fname, 'rb')
+                pickle.load(f)
+                f.close()
             engine = tenjin.Engine(**props)
             output = engine.render(':create', context)
             ok (output) == expected               # reloadable?
@@ -407,16 +420,22 @@ class EngineTest(object):
                 ok (fname).exists()                  # file created?
                 s = read_file(fname, 'r')            # read text file
                 ok (s.find(nullchar)) < 0            # text file?
-                #f = lambda: marshal.loads(s)
-                f = lambda: marshal.load(open(fname, 'rb'))
-                if python3:
-                    ok (f).raises(ValueError)        # non-marshal?
+                if JYTHON:
+                    continue
+                #fn = lambda: marshal.loads(s)
+                f = open(fname, 'rb')
+                fn = lambda: marshal.load(f)
+                try:
+                  if python3:
+                    ok (fn).raises(ValueError)        # non-marshal?
                     if sys.version_info[1] == 0:     # python 3.0
-                        ok (str(f.exception)) == "bad marshal data"
+                        ok (str(fn.exception)) == "bad marshal data"
                     else:                            # python 3.1 or later
-                        ok (str(f.exception)) == "bad marshal data (unknown type code)"
-                elif python2 and sys.version_info[1] >= 5:
-                    ok (f).raises(EOFError, "EOF read where object expected")  # non-marshal?
+                        ok (str(fn.exception)) == "bad marshal data (unknown type code)"
+                  elif python2 and sys.version_info[1] >= 5:
+                    ok (fn).raises(EOFError, "EOF read where object expected")  # non-marshal?
+                finally:
+                    f.close()
             engine = tenjin.Engine(**props)
             output = engine.render(':create', context)
             ok (output) == expected                  # reloadable?
@@ -425,7 +444,7 @@ class EngineTest(object):
 
     def test_cachefile_timestamp(self):
         """engine should clear cache not only template is newer but also template is older than cache."""
-        data = EngineTest.testdata['test_cachefile']
+        data = EngineTest._testdata['test_cachefile']
         filenames = { 'layout': 'layout.pyhtml',
                       'page': 'account_create.pyhtml',
                       'form': 'account_form.pyhtml',
@@ -473,7 +492,7 @@ class EngineTest(object):
         try:
             input = "<?py x = 10 ?>"
             template_name = "cachefile_delete.pyhtml"
-            open(template_name, 'w').write(input)
+            f = open(template_name, 'w'); f.write(input); f.close()
             storage = tenjin.MarshalCacheStorage()
             engine = tenjin.Engine(cache=storage)
             engine.render(template_name)
@@ -492,7 +511,7 @@ class EngineTest(object):
 
 
     def test_change_layout(self):
-        data = EngineTest.testdata['test_change_layout']
+        data = EngineTest._testdata['test_change_layout']
         ## setup
         tenjin.Engine.cache.clear()
         basenames = ['baselayout', 'customlayout', 'content']
@@ -510,7 +529,7 @@ class EngineTest(object):
 
 
     def test_context_scope(self):
-        data = EngineTest.testdata['test_context_scope']
+        data = EngineTest._testdata['test_context_scope']
         base = data['base']
         part = data['part']
         expected = data['expected']
@@ -528,7 +547,7 @@ class EngineTest(object):
 
 
     def test_template_args(self):
-        data = EngineTest.testdata['test_template_args']
+        data = EngineTest._testdata['test_template_args']
         content = data['content']
         expected = data['expected']
         context = data['context']
@@ -552,8 +571,8 @@ class EngineTest(object):
 
 
     def test__set_cache_storage(self):
-        if "default then Engine.cache is MarshalCacheStorage instance":
-            ok (tenjin.Engine.cache).is_a(tenjin.MarshalCacheStorage)
+        if "default then Engine.cache is TextCacheStorage instance":
+            ok (tenjin.Engine.cache).is_a(tenjin.TextCacheStorage)
         if "cache=True specified then use default cache object":
             engine = tenjin.Engine(cache=True)
             ok ('cache' in engine.__dict__) == False
@@ -592,7 +611,7 @@ class EngineTest(object):
 
 
     def test_cached_contents(self):
-        data = EngineTest.testdata['test_cached_contents']
+        data = EngineTest._testdata['test_cached_contents']
         def _test(filename, cachename, cachemode, input, expected_script, expected_args):
             if input:
                 write_file(filename, input)
@@ -600,10 +619,14 @@ class EngineTest(object):
             t = engine.get_template(filename)
             ok (t.args) == expected_args
             ok (t.script) == expected_script
-            import marshal
-            dct = marshal.load(open(filename + '.cache', 'rb'))
-            ok (dct['args']) == expected_args
-            ok (dct['script']) == expected_script
+            #import marshal
+            #f = open(filename + '.cache', 'rb')
+            #try:
+            #    dct = marshal.load(f)
+            #    ok (dct['args']) == expected_args
+            #    ok (dct['script']) == expected_script
+            #finally:
+            #    f.close()
         ##
         try:
             ## args=[x,y,z], cache=1
@@ -632,7 +655,7 @@ class EngineTest(object):
 
 
     def _test_template_path(self, keys):
-        data = EngineTest.testdata['test_template_path']
+        data = EngineTest._testdata['test_template_path']
         basedir = 'test_templates'
         try:
             os.mkdir(basedir)
@@ -681,7 +704,7 @@ class EngineTest(object):
 
 
     def test_preprocessor(self):
-        data = EngineTest.testdata['test_preprocessor']
+        data = EngineTest._testdata['test_preprocessor']
         try:
             basenames = ('form', 'create', 'update', 'layout', )
             filenames = []
@@ -723,9 +746,69 @@ class EngineTest(object):
         finally:
             _remove_files(filenames)
 
+    @_with_dummy_files
+    def test_pp(self):
+        if spec("'pp' paramater should be a list of preprocessor objects."):
+            pp1 = tenjin.TemplatePreprocessor()
+            pp2 = tenjin.TrimPreprocessor()
+            pp3 = tenjin.JavaScriptPreprocessor(type="text/javascript")
+            e = tenjin.Engine(pp=[pp1, pp2, pp3])
+            input = r"""
+<body>
+  <div>
+    <!-- #JS: render_items(items) -->
+    <ul>
+    <?js for (var i = 0; i < items.length; i++) { ?>
+      <li>${i}</li>
+    <?js } ?>
+    </ul>
+    <!-- #/JS -->
+  </div>
+  <script>#{{tenjin.JS_FUNC}}</script>
+</body>
+"""[1:]
+            expected = r"""
+<body>
+<div>
+<script type="text/javascript">function render_items(items){var _buf='';
+_buf+='<ul>\n';
+ for (var i = 0; i < items.length; i++) {
+_buf+='<li>'+_E(i)+'</li>\n';
+ }
+_buf+='</ul>\n';
+return _buf;};</script>
+</div>
+<script>function _S(x){return x==null?'':x;}
+function _E(x){return x==null?'':typeof(x)!=='string'?x:x.replace(/[&<>"']/g,_EF);}
+var _ET={'&':"&amp;",'<':"&lt;",'>':"&gt;",'"':"&quot;","'":"&#039;"};
+function _EF(c){return _ET[c];};</script>
+</body>
+"""[1:]
+            fname = 'tmp_123.pyhtml'
+            f = open(fname, 'w'); f.write(input); f.close()
+            try:
+                t = e.get_template(fname)
+                context = {}
+                output = e.render(fname, context)
+                ok (output) == expected
+            finally:
+                for x in glob(fname + '*'): os.unlink(x)
+
+    @test("#__init__(): creates TemplatePreprocessor object when 'preprocess' option is on.")
+    def _(self):
+        e = tenjin.Engine(preprocess=True)
+        ok (e.pp).is_a(list).length(1)
+        ok (e.pp[0]).is_a(tenjin.TemplatePreprocessor)
+
+    @test("#__init__(): creates TemplatePreprocessor object with 'preprocessorclass' class.")
+    def _(self):
+        e = tenjin.Engine(preprocess=True)
+        ok (e.pp[0].factory) == tenjin.Preprocessor
+        e = tenjin.Engine(preprocess=True, preprocessorclass=tenjin.SafePreprocessor)
+        ok (e.pp[0].factory) == tenjin.SafePreprocessor
 
     def test_include_with_kwargs(self):
-        data = EngineTest.testdata['test_include_with_kwargs']
+        data = EngineTest._testdata['test_include_with_kwargs']
         write_file('index.pyhtml', data['index_html'])
         write_file('sub.pyhtml', data['sub_html'])
         expected = data['expected']
@@ -833,7 +916,7 @@ class EngineTest(object):
 
     @_with_dummy_files
     def test__preprocess(self):
-        e1 = tenjin.Engine()
+        e1 = tenjin.Engine(preprocess=True)
         if spec("preprocess template and return result"):
             fpath = '_views/index.pyhtml'
             input, mtime = e1.loader.load(fpath)
@@ -865,7 +948,8 @@ class EngineTest(object):
             t._last_checked_at = None
             t.timestamp = os.path.getmtime(fpath)
             ok (e1._get_template_from_cache(cpath, fpath)).is_(t)
-            ok (t._last_checked_at).in_delta(time.time(), 0.001)
+            delta = JYTHON and 0.03 or 0.001
+            ok (t._last_checked_at).in_delta(time.time(), delta)
         if spec("if timestamp of template object is different from file, clear it"):
             t.timestamp = t.timestamp + 1
             t._last_checked_at = None
@@ -922,7 +1006,8 @@ class EngineTest(object):
             if spec("set timestamp and filename of template object."):
                 ok (t.timestamp) == os.path.getmtime(filepath)
                 ok (t.filename) == fpath
-                ok (t._last_checked_at).in_delta(time.time(), 0.005)
+                delta = JYTHON and 0.03 or 0.003
+                ok (t._last_checked_at).in_delta(time.time(), delta)
             if spec("save template object into cache."):
                 ok (cpath).exists()
                 ok (len(e1.cache.items)) == 1
@@ -955,6 +1040,18 @@ class EngineTest(object):
         if spec("add include() method into context data."):
             ok (ctx.get('include')) == (e.include)
 
+    @test("get_template(): ignores syntax error when compiling.")
+    def _(self):
+        input = """<p>${{foo}</p>"""
+        fname = "tmp_999.pyhtml"
+        f = open(fname, 'w'); f.write(input); f.close()
+        try:
+            e = tenjin.Engine()
+            def fn(): e.get_template(fname)
+            ok (fn).not_raise(SyntaxError)
+        finally:
+            for x in glob(fname + '*'): os.unlink(x)
+
 
 _DUMMY_VALUE = 'SOS'
 
