@@ -1,6 +1,6 @@
 ###
-### $Release: 1.0.2 $
-### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
+### $Release: 1.1.0 $
+### $Copyright: copyright(c) 2007-2012 kuwata-lab.com all rights reserved. $
 ### MIT License
 ###
 
@@ -12,6 +12,8 @@ from oktest import *
 import tenjin
 from tenjin.helpers import *
 
+JYTHON = hasattr(sys, 'JYTHON_JAR')
+
 lvars = "_extend=_buf.extend;_to_str=to_str;_escape=escape; "
 
 
@@ -34,7 +36,12 @@ else:
     if os.path.isdir(path):
         GAE_HOME = path
 
-if not GAE_HOME:
+
+if JYTHON:
+
+    sys.stderr.write("### skip GAE test (because on Jython)\n")
+
+elif not GAE_HOME:
 
     sys.stderr.write("###\n")
     sys.stderr.write("### WARNING:\n")
@@ -81,7 +88,7 @@ else:
 
         def test_01_init(self):
             import tenjin
-            ok (tenjin.Engine.cache).is_a(tenjin.MarshalCacheStorage)
+            ok (tenjin.Engine.cache).is_a(tenjin.TextCacheStorage)
             ok (tenjin.helpers.fragment_cache.store).is_a(tenjin.MemoryBaseStore)
             import tenjin.gae; tenjin.gae.init()
             if "called then change tenjin.Engine.cache to support GAE":
