@@ -1,17 +1,26 @@
 # -*- coding: utf-8 -*-
 ###
-### $Release: 1.0.2 $
-### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
+### $Release: 1.1.0 $
+### $Copyright: copyright(c) 2007-2012 kuwata-lab.com all rights reserved. $
 ###
 
 from oktest import ok, not_ok, run
-import os, traceback
+import sys, os, traceback, time
 import yaml
 
 from testcase_helper import *
 import tenjin
 from tenjin.helpers import escape, to_str
 
+PYPY   = hasattr(sys, 'pypy_version_info')
+JYTHON = hasattr(sys, 'JYTHON_JAR')
+
+def _errmsg(errmsg):
+    if PYPY:
+        if re.match(r"^name '.*' is not defined$", errmsg):
+            return "global " + errmsg
+    return errmsg
+
 filename = None
 for filename in ['../bin/pytenjin', 'bin/pytenjin']:
     if os.path.exists(filename):
@@ -68,7 +77,7 @@ print(''.join(_buf))
 """
 SOURCE2 = """_buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; _extend(('''<ul>\\r\\n''', ));\n\
 for item in ['<a&b>', '["c",'+"'d']"]:\n\
-    _extend(('''  <li>''', _to_str(item), '''\r\n\
+    _extend(('''  <li>''', _to_str(item), '''\\r\n\
       ''', _escape(_to_str(item)), '''</li>\\r\\n''', ));\n\
 #end\n\
 _extend(('''</ul>\\r\\n''', ));\n\
@@ -142,6 +151,24 @@ class MainTest(object):
             if expected:
                 expected = expected.replace(': unexpected indent', ': invalid syntax')
         #
+        if JYTHON:
+            if self._testMethodName == 'test_lint5':
+                expected = (
+                    ".test.pyhtml:5:4: mismatched input 'else' expecting DEDENT\n"
+                    "  5:     else\n"
+                    "        ^\n"
+                    )
+        #
+        if PYPY:
+            if errormsg:
+                errormsg = _errmsg(errormsg)
+            if self._testMethodName == 'test_lint5':
+                expected = (
+                    ".test.pyhtml:5:4: invalid syntax\n"
+                    "  5:     else\n"
+                    "        ^\n"
+                    )
+        #
         if filename is not False:
             if filename is None:
                 filename = '.test.pyhtml'
@@ -302,18 +329,33 @@ class MainTest(object):
             "#endfor\n"
             "_extend(('''</ul>\\n''', ));\n"
             )
+        expected_cache = r"""
+timestamp: %(timestamp)s
+args: title, items
+
+title = _context.get('title'); items = _context.get('items'); 
+_extend=_buf.extend;_to_str=to_str;_escape=escape; _extend(('''<h1>''', _escape(_to_str(title)), '''</h1>
+<ul>\n''', ));
+for item in items:
+    _extend(('''  <li>''', _escape(_to_str(item)), '''</li>\n''', ));
+#endfor
+_extend(('''</ul>\n''', ));
+"""[1:]  % {'timestamp': '%s.0' % int(time.time())}
         self.filename = 'test_cache1.pyhtml'
         cachename = self.filename + '.cache'
         try:
             self._test()
             ok (cachename).exists()
-            import marshal
-            dct = marshal.load(open(cachename, 'rb'))
-            ok (dct.get('args')) == ['title', 'items']
-            if   python2:  expected = "<type 'code'>"
-            elif python3:  expected = "<class 'code'>"
-            ok (str(type(dct.get('bytecode')))) == expected
-            ok (dct.get('script')) == script
+            #if not JYTHON:
+            #    import marshal
+            #    dct = marshal.load(open(cachename, 'rb'))
+            #    ok (dct.get('args')) == ['title', 'items']
+            #    if   python2:  expected = "<type 'code'>"
+            #    elif python3:  expected = "<class 'code'>"
+            #    ok (str(type(dct.get('bytecode')))) == expected
+            #    ok (dct.get('script')) == script
+            f = open(cachename); cached = f.read(); f.close()
+            ok (cached) == expected_cache
         finally:
             if os.path.exists(cachename):
                 os.unlink(cachename)
@@ -489,14 +531,23 @@ class MainTest(object):
         self._test()
 
     def test_dump(self):  # -d, -a dump
-        # create cache file
+        if JYTHON:
+            return
+        ## create cache file
+        #filename = '_test_dump.pyhtml'
+        #cachename = filename + '.cache'
+        #self.filename = filename
+        #self.input = INPUT
+        #self.expected = EXECUTED
+        #self.options = '-a render --cache=true'
+        #self._test()
+        #ok (cachename).exists()
+        ## create marshal cache file
         filename = '_test_dump.pyhtml'
         cachename = filename + '.cache'
-        self.filename = filename
-        self.input = INPUT
-        self.expected = EXECUTED
-        self.options = '-a render --cache=true'
-        self._test()
+        f = open(filename, 'w'); f.write(INPUT); f.close()
+        e = tenjin.Engine(cache=tenjin.MarshalCacheStorage())
+        e.get_template(filename)
         ok (cachename).exists()
         # dump test
         try:
@@ -852,6 +903,47 @@ class MainTest(object):
         finally:
             tenjin.Engine.templateclass = _backup
 
+    def test_pp(self):  # --pp=name,name2,...
+        input = r"""
+<div id="placeholder">
+  <!-- #JS: render_table(items) -->
+  <table>
+    <?js for (var i = 0; i < items.length; i++) { ?>
+    <tr>
+      <td>${item}</td>
+    </tr>
+    <?js } ?>
+  </table>
+  <!-- #/JS -->
+</div>
+<script>#{tenjin.JS_FUNC}</script>
+"""[1:]
+        output = r"""
+<div id="placeholder">
+<script>function render_table(items){var _buf='';
+_buf+='<table>\n';
+ for (var i = 0; i < items.length; i++) {
+_buf+='<tr>\n\
+<td>'+_E(item)+'</td>\n\
+</tr>\n';
+ }
+_buf+='</table>\n';
+return _buf;};</script>
+</div>
+<script>function _S(x){return x==null?'':x;}
+function _E(x){return x==null?'':typeof(x)!=='string'?x:x.replace(/[&<>"']/g,_EF);}
+var _ET={'&':"&amp;",'<':"&lt;",'>':"&gt;",'"':"&quot;","'":"&#039;"};
+function _EF(c){return _ET[c];};</script>
+"""[1:]
+        try:
+            self.options  = "--pp=Trim,PrefixedLine,JavaScript"
+            self.input = input
+            self.expected = output
+            self._test()
+        finally:
+            pass
+
+
 
 if __name__ == '__main__':
     run()
