@@ -1,9 +1,9 @@
 ###
-### $Release: 1.0.2 $
-### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
+### $Release: 1.1.0 $
+### $Copyright: copyright(c) 2007-2012 kuwata-lab.com all rights reserved. $
 ###
 
-from oktest import ok, not_ok, run, spec
+from oktest import ok, not_ok, run, spec, test
 from oktest.tracer import Tracer
 import sys, os, re
 
@@ -11,6 +11,15 @@ from testcase_helper import *
 import tenjin
 from tenjin.helpers import *
 
+PYPY   = hasattr(sys, 'pypy_version_info')
+JYTHON = hasattr(sys, 'JYTHON_JAR')
+
+def _errmsg(errmsg):
+    if PYPY:
+        if re.match(r"^name '.*' is not defined$", errmsg):
+            return "global " + errmsg
+    return errmsg
+
 
 class TemplateTest(object):
 
@@ -67,6 +76,14 @@ class TemplateTest(object):
                     source = source.replace("u'''", "'''").replace("u'", "'")
                     input  = input.replace("u'", "'")
         #
+        if JYTHON:
+            if self._testMethodName == 'test_syntaxerr1':
+                errormsg = r"mismatched input '\n' expecting COLON"
+        #
+        if PYPY:
+            if errormsg:
+                errormsg = _errmsg(errormsg)
+        #
         if exception:
             try:
                 template = tenjin.Template(**options)
@@ -284,7 +301,7 @@ _extend(('''</ul>\n''', ));
             t = tenjin.Template(None, input=input, tostrfunc='johnsmith')
             def f(): t.render({'name': 'Haruhi'})
             #ok (f).raises(TypeError, "'NoneType' object is not callable")
-            ok (f).raises(NameError, "name 'johnsmith' is not defined")
+            ok (f).raises(NameError, _errmsg("name 'johnsmith' is not defined"))
 
     def test_option_escapefunc(self):
         input = "<p>Hello ${name}!</p>"
@@ -312,8 +329,7 @@ _extend(('''</ul>\n''', ));
             t = tenjin.Template(None, input=input, escapefunc='kyonsmith')
             def f(): t.render({'name': 'Haruhi'})
             #ok (f).raises(TypeError, "'NoneType' object is not callable")
-            ok (f).raises(NameError, "name 'kyonsmith' is not defined")
-
+            ok (f).raises(NameError, _errmsg("name 'kyonsmith' is not defined"))
 
     def test_localvars_assignments_without_args_declaration(self):
         def _convert(input):
@@ -528,6 +544,19 @@ escape: ''', (var), '''\n''', ));
         #ok (cycle()).is_a(EscapedStr)
         #ok (cycle()).is_a(EscapedStr)
 
+    @test("'{}' is available in '${}' or '#{}', such as '${foo({'x':1})}'")
+    def _(self):
+        input = """
+<p>${f({'a':1})+g({'b':2})}</p>
+<p>#{f({'c':3})+g({'d':4})}</p>
+"""
+        expected = r"""_extend=_buf.extend;_to_str=to_str;_escape=escape; _extend(('''
+<p>''', _escape(_to_str(f({'a':1})+g({'b':2}))), '''</p>
+<p>''', _to_str(f({'c':3})+g({'d':4})), '''</p>\n''', ));
+"""
+        t = tenjin.Template()
+        script = t.convert(input)
+        ok (script) == expected
 
 
 
