@@ -1,6 +1,6 @@
 ###
-### $Release: 1.0.2 $
-### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
+### $Release: 1.1.0 $
+### $Copyright: copyright(c) 2007-2012 kuwata-lab.com all rights reserved. $
 ###
 
 #import unittest
@@ -24,6 +24,8 @@ elif python3:
     def _is_str(val):
         return isinstance(val, (str, bytes))
 
+JYTHON = hasattr(sys, 'JYTHON_JAR')
+
 
 def read_file(filename, mode='rb'):
     f = None
@@ -161,6 +163,9 @@ class TestCaseHelper:
 
     def load_testdata(filename, untabify=True):
         i = filename.rfind('.')
+        if JYTHON:
+            if filename[i-3:i] == '$py':
+                i -= 3
         if filename[i:] != '.yaml' and filename[i:] != '.yml':
             filename = filename[:i] + '.yaml'
         input = read_file(filename)
